/**
 Constants and functions used by the unSendMail CTRL manager and the panel
 vision/systemIntegrity/unMailCategoryConfig.pnl

 jean-charles.tournier@cern.ch
 CERN BE-ICS-FT
 December 2021
**/

const string UN_SENDMAIL_SEPARATOR = "~"; // Separator for the several field in Setting Dp
const string UN_SENDMAIL_DELIMITER = ";"; // Separaror for the Receivers

// Fields number when decoding _unSendMail.config.mail_settings
const int TYPE = 1;
const int DOMAIN = 2;
const int SMTP = 3;
const int SENDER = 4;
const int RECEIVER = 5;
const int LINK = 6;
const int MAX = 7;
const int USER = 8;
const int PWD = 9;
const int TLS = 10;

const string c_LemonLink = "http://abinf1/lemon/info.php?host=";
const string c_CosmosLink = "https://cosmos-grafana.cern.ch/d/jnBohMDGz/bec-details?refresh=30s&orgId=1&var-host=";

const int c_iMax = 250;

const string c_MailServer = "cernmx.cern.ch";
const string c_MailDomain = "cern.ch";


/*********************************************************
 * Simple function to hide/unhide password when storing it
 * in a DPE. As the password will have to be in clear text
 * anyway when sending the email, the goal is just to
 * not store the password in clear in the email config DPE.
 **********************************************************/
string unSendMail_hidePassword(string sPasswordClear, string sKey)
{
	blob bPasswordHidden;

	aes256Encrypt(sPasswordClear, sKey, bPasswordHidden);

	return (string)bPasswordHidden;
}

string unSendMail_unhidePassword(string sPasswordHidden, string sKey)
{
	string sPasswordClear;

	aes256Decrypt((blob)sPasswordHidden, sKey, sPasswordClear);

	return sPasswordClear;
}

/****************************************
 * Parse the string stored in _unSendMail.config.mail_settings
 ****************************************/
void unSendMail_parseConfigMailSettings(const dyn_string dsSetting,
																				dyn_string &dsType, dyn_string &dsSender, dyn_string &dsReceiver, dyn_string &dsDomain, dyn_string &dsSmtp,
																				dyn_string &dsLink, dyn_string &dsMax, dyn_string &dsUser, dyn_string &dsPassword, dyn_bool &dbUseSsl)
{
	_unSendMail_dynClearAll(dsType, dsSender, dsReceiver, dsDomain, dsSmtp, dsLink, dsMax, dsUser, dsPassword, dbUseSsl);
	int iLen = dynlen(dsSetting);
	for (int i = 1; i <= iLen; i++)
	{
		dyn_string dsField = strsplit(dsSetting[i], UN_SENDMAIL_SEPARATOR);
		string sLink, sMax;
		if (dynlen(dsField) >= 5)
		{
			dynAppend(dsType, dsField[TYPE]);

			string sSender;
			if (dsField[SENDER] == "none")
				sSender = "";
			else
				sSender = dsField[SENDER];
			dynAppend(dsSender, sSender);

			string sReceiver;
			if (dsField[RECEIVER] == "none")
				sReceiver = "";
			else
				sReceiver = dsField[RECEIVER];
			dynAppend(dsReceiver, sReceiver);

			dynAppend(dsDomain, dsField[DOMAIN]);
			dynAppend(dsSmtp, dsField[SMTP]);
			if (dynlen(dsField) > 5)
			{
				// keep compatibility with old version
				sLink = dsField[LINK];
				if (dynlen(dsField) > 6)
					sMax = dsField[MAX];
				else
					sMax = c_iMax;
			}
			else
			{
				sLink = "";
				sMax = c_iMax;
			}

			dynAppend(dsLink, sLink);
			dynAppend(dsMax, sMax);

			if (dynlen(dsField) >= 10)
			{
				dynAppend(dsUser, dsField[USER]);
				dynAppend(dsPassword, unSendMail_unhidePassword(dsField[PWD], dsField[USER]));
				dynAppend(dbUseSsl, dsField[TLS]);
			}
			else
			{
				dynAppend(dsUser, "");
				dynAppend(dsPassword, "");
				dynAppend(dbUseSsl, FALSE);
			}
		}
		else
		{
			// default values
			dynAppend(dsType, dsField[TYPE]);
			dynAppend(dsSender, "");
			dynAppend(dsReceiver, "");
			dynAppend(dsDomain, c_MailDomain);
			dynAppend(dsSmtp, c_MailServer);
			dynAppend(dsLink, unSendMail_getHTMLLink());
			dynAppend(dsMax, c_iMax);
			dynAppend(dsUser, "");
			dynAppend(dsPassword, "");
			dynAppend(dbUseSsl, FALSE);
		}
	}
}

/*********************************
 * Simple helper function to 
 * clear multiple dyn_* at once 
 ********************************/
void _unSendMail_dynClearAll( ...){
	va_list parameters;
	int iNum = va_start(parameters);
	for( int i=1; i<=iNum; i++){
		switch (getType(parameters[i]))
		{
			case DYN_BIT32_VAR:
			case DYN_BOOL_VAR:
			case DYN_BLOB_VAR:
			case DYN_CHAR_VAR:
			case DYN_CHAR_VAR:
			case DYN_FLOAT_VAR:
			case DYN_INT_VAR:
			case DYN_LANGSTRING_VAR:
			case DYN_STRING_VAR:
			case DYN_TIME_VAR:
			case DYN_UINT_VAR:
			case DYN_ATIME_VAR:
				dynClear(parameters[i]);
				break;
			default:
				throwError(makeError("",PRIO_SEVERE,ERR_CONTROL,0,"["+__FUNCTION__+"] Trying to call dynClear on a non-dyn variable"));
		}
	}
	va_end(parameters);
}

/****************************
 * Get the cosmos link for the current
 * data server.
 ****************************/
string unSendMail_getHTMLLink()
{
  string sLink, sHost, sIPAddress;

  unGenericDpFunctions_getHostName(getSystemName(), sHost, sIPAddress);  
  sLink = c_CosmosLink + strtoupper(sHost);
  return sLink;
}